@@ -36,8 +36,9 @@ class Request(object):
         "SERVER_SOFTWARE": "gunicorn/%s" % __version__
     }
 
-    def __init__(self, socket, client_address, server_address, debug=False):
-        self.debug = debug
+    def __init__(self, socket, client_address, server_address, conf):
+        self.debug = conf['debug']
+        self.conf = conf
         self.socket = socket
     
         self.client_address = client_address
@@ -48,11 +49,11 @@ class Request(object):
         self.parser = Parser()
         self.start_response_called = False
         self.log = logging.getLogger(__name__)
+        self.response_chunked = False
 
     def read(self):
         environ = {}
         headers = []
-        buf = ""
         buf = read_partial(self.socket, CHUNK_SIZE)
         i = self.parser.filter_headers(headers, buf)
         if i == -1 and buf:
@@ -61,7 +62,8 @@ class Request(object):
                 if not data: break
                 buf += data
                 i = self.parser.filter_headers(headers, buf)
-                if i != -1: break
+                if i != -1:
+                    break            
 
         self.log.debug("%s", self.parser.status)
         self.log.debug("Headers:\n%s" % headers)
@@ -72,7 +74,7 @@ class Request(object):
         if not self.parser.content_len and not self.parser.is_chunked:
             wsgi_input = StringIO.StringIO()
         else:
-            wsgi_input = TeeInput(self.socket, self.parser, buf[i:])
+            wsgi_input = TeeInput(self.socket, self.parser, buf[i:], self.conf)
                 
         # This value should evaluate true if an equivalent application
         # object may be simultaneously invoked by another process, and
@@ -154,6 +156,9 @@ class Request(object):
         self.response_status = status
         for name, value in response_headers:
             name = normalize_name(name)
+            if name == "Transfer-Encoding":
+                if value.lower() == "chunked":
+                    self.response_chunked = True
             if not isinstance(value, basestring):
                 value = str(value)
             self.response_headers.append((name, value.strip()))     
