@@ -19,7 +19,8 @@ from gunicorn.util import MAX_BODY, CHUNK_SIZE, read_partial
 
 class TeeInput(object):
     
-    def __init__(self, socket, parser, buf):
+    def __init__(self, socket, parser, buf, conf):
+        self.conf = conf
         self.buf = buf
         self.parser = parser
         self.socket = socket
@@ -28,26 +29,36 @@ class TeeInput(object):
         if self._len and self._len < MAX_BODY:
             self.tmp = StringIO.StringIO()
         else:
-            self.tmp = tempfile.TemporaryFile()
+            self.tmp = tempfile.TemporaryFile(
+                            dir=self.conf['tmp_upload_dir'])
             
         if len(buf) > 0:
             chunk, self.buf = parser.filter_body(buf)
             if chunk:
                 self.tmp.write(chunk)
-                self.tmp.seek(0)
             self._finalize()
+            self.tmp.seek(0)
         
     @property
     def len(self):
         if self._len: return self._len
+        
         if self._is_socket:
             pos = self.tmp.tell()
             while True:
+                self.tmp.seek(self._tmp_size())
                 if not self._tee(CHUNK_SIZE):
                     break
             self.tmp.seek(pos)
         self._len = self._tmp_size()
         return self._len
+        
+    def seek(self, offset, whence=0):
+        if self._is_socket:
+            while True:
+                if not self._tee(CHUNK_SIZE):
+                    break
+        self.tmp.seek(offset, whence)
 
     def flush(self):
         self.tmp.flush()
@@ -84,12 +95,18 @@ class TeeInput(object):
         
         # now we can get line
         line = self.tmp.readline()
-        if size > 0 and len(line) < size:
-            self.tmp.seek(orig_size)
+        i = line.find("\n")
+        if i == -1:
             while True:
+                orig_size = self.tmp.tell()
                 if not self._tee(CHUNK_SIZE):
-                    self.tmp.seek(orig_size)
-                    return self.temp.readline(size)
+                    break
+                self.tmp.seek(orig_size)
+                line = self.tmp.readline()
+                i = line.find("\n")
+                if i != -1: 
+                    break
+                    
         return line
        
     def readlines(self, sizehint=0):
@@ -116,14 +133,18 @@ class TeeInput(object):
 
     def _tee(self, length):
         """ fetch partial body"""
-        while not self.parser.body_eof():
-            data = read_partial(self.socket, length)
-            self.buf += data
+        while True:
             chunk, self.buf = self.parser.filter_body(self.buf)
             if chunk:
                 self.tmp.write(chunk)
+                self.tmp.flush()
                 self.tmp.seek(0, os.SEEK_END)
-                return chunk        
+                return chunk
+            
+            if self.parser.body_eof():
+                break
+            
+            self.buf = read_partial(self.socket, length, self.buf)
         self._finalize()
         return ""
         
@@ -131,12 +152,6 @@ class TeeInput(object):
         """ here we wil fetch final trailers
         if any."""
         if self.parser.body_eof():
-            # handle trailing headers
-            if self.parser.is_chunked and self._is_socket:
-                while not self.parser.trailing_header(self.buf):
-                    data = read_partial(self.socket, CHUNK_SIZE)
-                    if not data: break
-                    self.buf += data
             del self.buf
             self._is_socket = False
             
@@ -149,6 +164,10 @@ class TeeInput(object):
     def _ensure_length(self, buf, length):
         if not buf or not self._len:
             return buf
-        while len(buf) < length and self.len != self.tmp.tell():
-            buf += self._tee(length - len(buf))
+        while True:
+            if len(buf) >= length: 
+                break
+            data = self._tee(length - len(buf))
+            if not data: break
+            buf += data
         return buf
