@@ -3,12 +3,11 @@
 # This file is part of gunicorn released under the MIT license. 
 # See the NOTICE for more information.
 
+import array
 import ctypes
-import errno
 import fcntl
 import os
 import resource
-import select
 import socket
 import textwrap
 import time
@@ -91,49 +90,38 @@ def close(sock):
     except socket.error:
         pass
   
-def read_partial(sock, length):
-    while True:
-        try:
-            ret = select.select([sock.fileno()], [], [], 0)
-            if ret[0]: break
-        except select.error, e:
-            if e[0] == errno.EINTR:
-                continue
-            raise
-    data = sock.recv(length)
-    return data
+def read_partial(sock, length, buf=None):
+    tmp_buf = array.array("c", '\0' * length)
+    l = sock.recv_into(tmp_buf, length)
+    
+    if not buf:
+        return tmp_buf[:l]
+    
+    return buf + tmp_buf[:l]
 
-def write(sock, data):
-    buf = ""
-    buf += data
-    i = 0
-    while buf:
-        try:
-            bytes = sock.send(buf)
-            if bytes < len(buf):
-                buf = buf[bytes:]
-                continue
-            return len(data)
-        except socket.error, e:
-            if e[0] in (errno.EWOULDBLOCK, errno.EAGAIN):
-                break
-            raise
-        i += 1
-        
-def write_nonblock(sock, data):
+def write_chunk(sock, data):
+    chunk = "".join(("%X\r\n" % len(data), data, "\r\n"))
+    sock.sendall(chunk)
+    
+def write(sock, data, chunked=False):
+    if chunked:
+        return write_chunk(sock, data)
+    sock.sendall(data)
+
+def write_nonblock(sock, data, chunked=False):
     timeout = sock.gettimeout()
     if timeout != 0.0:
         try:
             sock.setblocking(0)
-            return write(sock, data)
+            return write(sock, data, chunked)
         finally:
             sock.setblocking(1)
     else:
-        return write(sock, data)
+        return write(sock, data, chunked)
     
-def writelines(sock, lines):
+def writelines(sock, lines, chunked=False):
     for line in list(lines):
-        write(sock, line)
+        write(sock, line, chunked)
 
 def write_error(sock, msg):
     html = textwrap.dedent("""\
